import os
import platform

import cv2
import requests

from mower import branch, commit
from mower.utils import config
from mower.utils.graph import SceneGraphSolver
from mower.utils.image import crop2content, cropimg, loadres, thres2
from mower.utils.log import logger
from mower.utils.rapidocr import ocr_rec
from mower.utils.recognize import Scene
from mower.utils.vector import sa, va


class CreditSolver(SceneGraphSolver):
    solver_name = "访问好友"

    def run(self) -> None:
        self.wait_times = 5
        self.info_uploaded = not config.conf.telemetry
        return super().run()

    def upload_data(self):
        img = cropimg(config.recog.gray, ((1390, 35), (1920, 95)))
        img = thres2(img, 250)
        img = crop2content(img)
        post_data = {
            "name": ocr_rec(img),
            "processor": platform.processor(),
            "cpu_count": os.cpu_count(),
            "emulator": config.conf.emulator.name,
            "screencap": config.conf.screencap_strategy,
            "avg_time": round(config.screenshot_avg),
            "branch": branch,
            "commit": commit,
        }
        try:
            r = requests.post("https://mower-ng.zhaozuohong.vip/stat", json=post_data)
            if (ratio := int(r.text)) >= 80:
                msg = f"您的截图速度击败了{r.text}%以上的电脑，特此授予五星级神机称号！"
                color = "success"
            elif ratio >= 30:
                msg = f"您的截图速度击败了{r.text}%的电脑，请继续保持哦！"
                color = "info"
            else:
                msg = f"您的截图速度过慢，只击败了{r.text}%的电脑，建议立即优化！"
                color = "warning"
            logger.info(msg)
            ws_data = {"type": "screenshot", "color": color, "msg": msg}
            config.screenshot_notification = ws_data
            config.ws_queue.put(ws_data)
        except Exception as e:
            logger.exception(e)
        self.info_uploaded = True

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.FRIEND_LIST:
            top_left = 1460, 220
            img = cropimg(config.recog.gray, (top_left, (1800, 1000)))
            img = thres2(img, 245)
            tpl = loadres("friend_visit", True)
            result = cv2.matchTemplate(img, tpl, cv2.TM_SQDIFF_NORMED)
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
            h, w = tpl.shape
            scope = min_loc, va(min_loc, (w, h))
            scope = sa(scope, top_left)
            logger.debug(f"{min_val=}, {scope=}")
            if min_val > 0.5:
                return
            self.tap(scope, update=False)
            if not self.info_uploaded:
                self.upload_data()
        elif self.find("visit_limit"):
            logger.info("今日参与交流已达上限")
            return True
        elif scene == Scene.FRIEND_VISITING:
            if clue_next := self.find("clue_next"):
                x, y = self.get_pos(clue_next, x_rate=0.5, y_rate=0.85)
                if abs(config.recog.hsv[y][x][0] - 12) < 3:
                    self.wait_times = 5
                    self.tap(clue_next)
                else:
                    return True
            else:
                if self.wait_times > 0:
                    self.wait_times -= 1
                    self.sleep()
                else:
                    return True
        else:
            self.scene_graph_step(Scene.FRIEND_LIST)
